@@ -1,7 +1,7 @@
-/* $Id: control.cc 1880 2014-07-10 12:35:32Z dreibh $
+/* $Id: control.cc 1933 2014-11-17 07:41:37Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2014 by Thomas Dreibholz
+ * Copyright (C) 2009-2015 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
 {
    // ====== Sent NETPERFMETER_ADD_FLOW to remote node ======================
    const size_t                addFlowMsgSize = sizeof(NetPerfMeterAddFlowMessage) +
-                                                   (sizeof(uint32_t) * flow->getTrafficSpec().OnOffEvents.size());
+                                                   (sizeof(NetPerfMeterOnOffEvent) * flow->getTrafficSpec().OnOffEvents.size());
    char                        addFlowMsgBuffer[addFlowMsgSize];
    NetPerfMeterAddFlowMessage* addFlowMsg = (NetPerfMeterAddFlowMessage*)&addFlowMsgBuffer;
    addFlowMsg->Header.Type   = NETPERFMETER_ADD_FLOW;
@@ -160,6 +160,9 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    if(flow->getTrafficSpec().NoDelay == true) {
       addFlowMsg->Header.Flags |= NPMAFF_NODELAY;
    }
+   if(flow->getTrafficSpec().RepeatOnOff == true) {
+      addFlowMsg->Header.Flags |= NPMAFF_REPEATONOFF;
+   }
 
    addFlowMsg->Header.Length = htons(addFlowMsgSize);
    addFlowMsg->MeasurementID = hton64(flow->getMeasurementID());
@@ -186,14 +189,26 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
 
    addFlowMsg->OnOffEvents   = htons(flow->getTrafficSpec().OnOffEvents.size());
    size_t i = 0;
-   for(std::set<unsigned int>::iterator iterator = flow->getTrafficSpec().OnOffEvents.begin();
+   for(std::vector<OnOffEvent>::const_iterator iterator = flow->getTrafficSpec().OnOffEvents.begin();
        iterator != flow->getTrafficSpec().OnOffEvents.end();iterator++, i++) {
-      addFlowMsg->OnOffEvent[i] = htonl(*iterator);
+      addFlowMsg->OnOffEvent[i].RandNumGen = (*iterator).RandNumGen;
+      addFlowMsg->OnOffEvent[i].Flags      = 0x00;
+      if((*iterator).RelativeTime == true) {
+         addFlowMsg->OnOffEvent[i].Flags |= NPOOEF_RELTIME;
+      }
+      addFlowMsg->OnOffEvent[i].pad = 0x00;
+      for(size_t j = 0; j < NETPERFMETER_RNG_INPUT_PARAMETERS; j++) {
+         addFlowMsg->OnOffEvent[i].ValueArray[j] = doubleToNetwork((*iterator).ValueArray[j]);
+      }
    }
    memset((char*)&addFlowMsg->Description, 0, sizeof(addFlowMsg->Description));
    strncpy((char*)&addFlowMsg->Description, flow->getTrafficSpec().Description.c_str(),
            std::min(sizeof(addFlowMsg->Description), flow->getTrafficSpec().Description.size()));
 
+   memset((char*)&addFlowMsg->CongestionControl, 0, sizeof(addFlowMsg->CongestionControl));
+   strncpy((char*)&addFlowMsg->CongestionControl, flow->getTrafficSpec().CongestionControl.c_str(),
+           std::min(sizeof(addFlowMsg->CongestionControl), flow->getTrafficSpec().CongestionControl.size()));
+
    addFlowMsg->NDiffPorts = htons(flow->getTrafficSpec().NDiffPorts);
    memset((char*)&addFlowMsg->PathMgr, 0, sizeof(addFlowMsg->PathMgr));
    strncpy((char*)&addFlowMsg->PathMgr, flow->getTrafficSpec().PathMgr.c_str(),
@@ -353,6 +368,7 @@ bool performNetPerfMeterStart(MessageReader*         messageReader,
          fprintf(configFile, "FLOW%u_ORDERED=%f\n",                          flow->getFlowID(), flow->getTrafficSpec().OrderedMode);
          fprintf(configFile, "FLOW%u_NODELAY=\"%s\"\n",                      flow->getFlowID(), (flow->getTrafficSpec().NoDelay == true) ? "on" : "off");
          fprintf(configFile, "FLOW%u_DEBUG=\"%s\"\n",                        flow->getFlowID(), (flow->getTrafficSpec().Debug == true) ? "on" : "off");
+         fprintf(configFile, "FLOW%u_CC=\"%s\"\n",                           flow->getFlowID(), flow->getTrafficSpec().CongestionControl.c_str());
          fprintf(configFile, "FLOW%u_PATHMGR=\"%s\"\n",                      flow->getFlowID(), flow->getTrafficSpec().PathMgr.c_str());
          fprintf(configFile, "FLOW%u_NDIFFPORTS=%u\n",                       flow->getFlowID(), flow->getTrafficSpec().NDiffPorts);
          fprintf(configFile, "FLOW%u_VECTOR_ACTIVE_NODE=\"%s\"\n",           flow->getFlowID(), flow->getVectorFile().getName().c_str());
@@ -754,20 +770,32 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
       trafficSpec.SndBufferSize            = ntohl(addFlowMsg->SndBufferSize);
       trafficSpec.OrderedMode              = ntohl(addFlowMsg->OrderedMode)  / (double)0xffffffff;
       trafficSpec.ReliableMode             = ntohl(addFlowMsg->ReliableMode) / (double)0xffffffff;
+      trafficSpec.CMT                      = addFlowMsg->CMT;
+      trafficSpec.CCID                     = addFlowMsg->CCID;
+      trafficSpec.NoDelay                  = (addFlowMsg->Header.Flags & NPMAFF_NODELAY);
+      trafficSpec.Debug                    = (addFlowMsg->Header.Flags & NPMAFF_DEBUG);
+      trafficSpec.RepeatOnOff              = (addFlowMsg->Header.Flags & NPMAFF_REPEATONOFF);
       trafficSpec.RetransmissionTrials     = ntohl(addFlowMsg->RetransmissionTrials) & ~NPMAF_RTX_TRIALS_IN_MILLISECONDS;
       trafficSpec.RetransmissionTrialsInMS = (ntohl(addFlowMsg->RetransmissionTrials) & NPMAF_RTX_TRIALS_IN_MILLISECONDS);
       if( (trafficSpec.RetransmissionTrialsInMS) && (trafficSpec.RetransmissionTrials == 0x7fffffff) ) {
          trafficSpec.RetransmissionTrials = ~0;
       }
-      const uint32_t* events = (const uint32_t*)&addFlowMsg->OnOffEvent;
-      for(size_t i = 0;i < startStopEvents;i++) {
-         trafficSpec.OnOffEvents.insert(ntohl(events[i]));
-      }
-      trafficSpec.CMT  = addFlowMsg->CMT;
-      trafficSpec.CCID = addFlowMsg->CCID;
 
-      trafficSpec.NoDelay = (addFlowMsg->Header.Flags & NPMAFF_NODELAY);
-      trafficSpec.Debug   = (addFlowMsg->Header.Flags & NPMAFF_DEBUG);
+      const NetPerfMeterOnOffEvent* event = (const NetPerfMeterOnOffEvent*)&addFlowMsg->OnOffEvent;
+      for(size_t i = 0;i < startStopEvents;i++) {
+         OnOffEvent newEvent;
+         newEvent.RandNumGen   = event[i].RandNumGen;
+         newEvent.RelativeTime = (event[i].Flags & NPOOEF_RELTIME);
+         for(size_t j = 0; j < NETPERFMETER_RNG_INPUT_PARAMETERS; j++) {
+            newEvent.ValueArray[j] = networkToDouble(event[i].ValueArray[j]);
+         }
+         trafficSpec.OnOffEvents.push_back(newEvent);
+      }
+
+      char congestionControl[sizeof(addFlowMsg->CongestionControl) + 1];
+      memcpy((char*)&congestionControl, (const char*)&addFlowMsg->CongestionControl, sizeof(addFlowMsg->CongestionControl));
+      congestionControl[sizeof(addFlowMsg->CongestionControl)] = 0x00;
+      trafficSpec.CongestionControl = std::string(congestionControl);
 
       char pathMgr[sizeof(addFlowMsg->PathMgr) + 1];
       memcpy((char*)&pathMgr, (const char*)&addFlowMsg->PathMgr, sizeof(addFlowMsg->PathMgr));
