@@ -1,8 +1,8 @@
-/* $Id: flow.cc 1880 2014-07-10 12:35:32Z dreibh $
+/* $Id: flow.cc 1933 2014-11-17 07:41:37Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2014 by Thomas Dreibholz
+ * Copyright (C) 2009-2015 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,9 @@
 #include <assert.h>
 #include <math.h>
 #include <netinet/tcp.h>
+#include <set>
+
+#include <set>
 
 
 // Flow Manager Singleton object
@@ -204,7 +207,8 @@ bool FlowManager::startMeasurement(const uint64_t           measurementID,
             if(flow->MeasurementID == measurementID) {
                flow->setMeasurement(measurement);
                if(flow->SocketDescriptor >= 0) {
-                  flow->BaseTime     = now;
+                  flow->TimeBase     = now;
+                  flow->TimeOffset    = 0;
                   flow->InputStatus  = Flow::On;
                   flow->OutputStatus = (flow->TrafficSpec.OnOffEvents.size() > 0) ? Flow::Off : Flow::On;
                   if(printFlows) {
@@ -852,7 +856,7 @@ Flow::Flow(const uint64_t         measurementID,
 
    InputStatus                   = WaitingForStartup;
    OutputStatus                  = WaitingForStartup;
-   BaseTime                      = getMicroTime();
+   TimeBase                      = getMicroTime();
 
    TrafficSpec                   = trafficSpec;
 
@@ -865,6 +869,7 @@ Flow::Flow(const uint64_t         measurementID,
    LastOutboundSeqNumber         = ~0;
    LastOutboundFrameID           = ~0;
    NextStatusChangeEvent         = ~0ULL;
+   OnOffEventPointer             = 0;
    unlock();
 
    FlowManager::getFlowManager()->addFlow(this);
@@ -1095,14 +1100,28 @@ unsigned long long Flow::scheduleNextStatusChangeEvent(const unsigned long long
 {
    lock();
 
-   std::set<unsigned int>::iterator first = TrafficSpec.OnOffEvents.begin();
-   if((OutputStatus != WaitingForStartup) && (first != TrafficSpec.OnOffEvents.end())) {
-      const unsigned int relNextEvent = *first;
-      const unsigned long long absNextEvent = BaseTime + (1000ULL * relNextEvent);
-      NextStatusChangeEvent = absNextEvent;
-   }
-   else {
+   if(OnOffEventPointer >= TrafficSpec.OnOffEvents.size()) {
       NextStatusChangeEvent = ~0ULL;
+      if(TrafficSpec.RepeatOnOff == true) {
+//          printf("Rewinding flow #%u\n", FlowID);
+         OnOffEventPointer = 0;
+      }
+   }
+
+   if(OnOffEventPointer < TrafficSpec.OnOffEvents.size()) {
+      const OnOffEvent&        event        = TrafficSpec.OnOffEvents[OnOffEventPointer];
+      const unsigned long long relNextEvent = (const unsigned long long)rint(1000000.0 * getRandomValue((const double*)&event.ValueArray, event.RandNumGen));
+      const unsigned long long absNextEvent = TimeBase + TimeOffset + relNextEvent;
+
+      TimeOffset            = TimeOffset + relNextEvent;
+      NextStatusChangeEvent = absNextEvent;
+
+//       printf("Schedule flow #%u:\trel=%llu\tin=%lld\n",
+//              FlowID, relNextEvent, (long long)absNextEvent - (long long)now);
+
+      if((long long)absNextEvent - (long long)now < -10000000) {
+         std::cerr << "WARNING: Schedule is more than 10s behind clock time! Check on/off parameters!" << std::endl;
+      }
    }
 
    unlock();
@@ -1115,8 +1134,7 @@ void Flow::handleStatusChangeEvent(const unsigned long long now)
 {
    lock();
    if(NextStatusChangeEvent <= now) {
-      std::set<unsigned int>::iterator first = TrafficSpec.OnOffEvents.begin();
-      assert(first != TrafficSpec.OnOffEvents.end());
+      assert(OnOffEventPointer < TrafficSpec.OnOffEvents.size());
 
       if(OutputStatus == Off) {
          OutputStatus = On;
@@ -1128,9 +1146,9 @@ void Flow::handleStatusChangeEvent(const unsigned long long now)
          assert(false);
       }
 
-      TrafficSpec.OnOffEvents.erase(first);
+      OnOffEventPointer++;
+      scheduleNextStatusChangeEvent(now);
    }
-   scheduleNextStatusChangeEvent(now);
    unlock();
 }
 
@@ -1140,13 +1158,14 @@ void Flow::run()
 {
    signal(SIGPIPE, SIG_IGN);
 
+   scheduleNextStatusChangeEvent(getMicroTime());
+
    bool result = true;
    do {
       // ====== Schedule next status change event ===========================
       unsigned long long       now              = getMicroTime();
-      const unsigned long long nextStatusChange = scheduleNextStatusChangeEvent(now);
       const unsigned long long nextTransmission = scheduleNextTransmissionEvent();
-      unsigned long long       nextEvent        = std::min(nextStatusChange, nextTransmission);
+      unsigned long long       nextEvent        = std::min(NextStatusChangeEvent, nextTransmission);
 
       // ====== Wait until there is something to do =========================
       if(nextEvent > now) {
@@ -1190,7 +1209,7 @@ void Flow::run()
       }
 
       // ====== Handle status changes =======================================
-      if(nextStatusChange <= now) {
+      if(NextStatusChangeEvent <= now) {
          handleStatusChangeEvent(now);
       }
    } while( (result == true) && (!isStopping()) );
@@ -1249,6 +1268,19 @@ bool Flow::configureSocket(const int socketDescriptor)
          }
 #endif
       }
+#ifndef __linux__
+#warning Congestion Control selection is currently only available on Linux!
+#else
+      const char* congestionControl = TrafficSpec.CongestionControl.c_str();
+      if(strcmp(congestionControl, "default") != 0) {
+         if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_CONGESTION, congestionControl, strlen(congestionControl)) < 0) {
+            std::cerr << "ERROR: Failed to set TCP_CONGESTION ("
+                      << congestionControl << ") on socket - "
+                      << strerror(errno) << "!" << std::endl;
+            return(false);
+         }
+      }
+#endif
    }
    else if(TrafficSpec.Protocol == IPPROTO_SCTP) {
       if (TrafficSpec.NoDelay) {
