@@ -1,8 +1,8 @@
-/* $Id: flowtrafficspec.cc 1877 2014-07-09 13:44:24Z dreibh $
+/* $Id: flowtrafficspec.cc 1933 2014-11-17 07:41:37Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2014 by Thomas Dreibholz
+ * Copyright (C) 2009-2015 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,12 +57,16 @@ void FlowTrafficSpec::showEntry(std::ostream& os,
          snprintf((char*)&str, sizeof(str), "%1.6lf +/- %1.3lf%% (uniform)",
                   valueArray[0], 100.0 * valueArray[1]);
        break;
+      case RANDOM_PARETO:
+         snprintf((char*)&str, sizeof(str), "m=%1.6lf, k=%1.6lf%% (pareto)",
+                  valueArray[0], valueArray[1]);
+       break;
       default:
          snprintf((char*)&str, sizeof(str), "unknown?!");
        break;
    }
 
-   os << str;
+   os << str << " ";
 }
 
 
@@ -106,36 +110,32 @@ void FlowTrafficSpec::print(std::ostream& os) const
       }
       os << std::endl;
    }
-   os << "      - Start/Stop:          { ";
+
+   os << "      - On/Off:              {";
    if(OnOffEvents.size() > 0) {
       bool start = true;
-      for(std::set<unsigned int>::iterator iterator = OnOffEvents.begin();
+      for(std::vector<OnOffEvent>::const_iterator iterator = OnOffEvents.begin();
           iterator != OnOffEvents.end();iterator++) {
-         if(start) {
-            os << "*" << (*iterator / 1000.0) << " ";
-         }
-         else {
-            os << "~" << (*iterator / 1000.0) << " ";
-         }
+         os << std::endl << "         " << ((start == true) ? "* " : "~ ");
+         showEntry(os, (const double*)&(*iterator).ValueArray, (*iterator).RandNumGen);
          start = !start;
       }
    }
    else {
       os << "*0 ";
    }
-   os << "}" << std::endl;
+   os << "}" << std::endl
+      << "      - Repeat On/Off:       " << ((RepeatOnOff == true) ? "yes" : "no") << std::endl;
 
    os << "      - Error on Abort:      "
       << ((ErrorOnAbort == true) ? "yes" : "no") << std::endl
       << "      - Debug:               "
       << ((Debug == true) ? "yes" : "no") << std::endl
       << "      - No Delay:            "
-      << ((NoDelay == true) ? "yes" : "no") << std::endl;
-
-//    if(Protocol == IPPROTO_MPTCP) {
-      os << "      Number of Diff. Ports: " << NDiffPorts << std::endl
-         << "      Path Manager:          " << PathMgr    << std::endl;
-//    }
+      << ((NoDelay == true) ? "yes" : "no") << std::endl
+      << "      Congestion Control:    " << CongestionControl << std::endl
+      << "      Number of Diff. Ports: " << NDiffPorts        << std::endl
+      << "      Path Manager:          " << PathMgr           << std::endl;
 
    os << "      - CMT:                 #" << (unsigned int)CMT << " ";
    switch(CMT) {
@@ -174,8 +174,10 @@ void FlowTrafficSpec::reset()
    ErrorOnAbort             = true;
    Debug                    = false;
    NoDelay                  = false;
+   RepeatOnOff              = false;
    NDiffPorts               = 4;
    PathMgr                  = "default";
+   CongestionControl        = "default";
    CMT                      = 0x00;
    CCID                     = 0x00;
    for(size_t i = 0;i < NETPERFMETER_RNG_INPUT_PARAMETERS;i++) {
