@@ -1,8 +1,8 @@
-/* $Id: netperfmeter.cc 1877 2014-07-09 13:44:24Z dreibh $
+/* $Id: netperfmeter.cc 1933 2014-11-17 07:41:37Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2014 by Thomas Dreibholz
+ * Copyright (C) 2009-2015 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -161,7 +161,13 @@ static const char* parseNextEntry(const char* parameters,
    }
    else if(sscanf(parameters, "uniform%lf,%lf%n", &valueArray[0], &valueArray[1], &n) == 2) {
       *rng = RANDOM_UNIFORM;
-      // printf("%lf +/- %lf%%\n",valueArray[0],valueArray[1]*100);
+   }
+   else if(sscanf(parameters, "pareto%lf,%lf%n", &valueArray[0], &valueArray[1], &n) == 2) {
+      *rng = RANDOM_PARETO;
+   }
+   else if(sscanf(parameters, "%lf%n", &valueArray[0], &n) == 1) {
+      // shortcut: number interpreted as constant
+      *rng = RANDOM_CONSTANT;
    }
    else {
       cerr << "ERROR: Invalid parameters " << parameters << endl;
@@ -333,28 +339,48 @@ static const char* parseTrafficSpecOption(const char*      parameters,
       n = 12 + strlen(description);
    }
    else if(strncmp(parameters,"onoff=", 6) == 0) {
-      unsigned int lastEvent = 0;
-      size_t       m         = 5;
+      size_t m = 5;
+      trafficSpec.RepeatOnOff = false;
       while( (parameters[m] != 0x00) && (parameters[m] != ':') ) {
          m++;
-         double       dblValue;
-         unsigned int onoff;
-         if(sscanf((const char*)&parameters[m], "+%lf%n", &dblValue, &n) == 1) {
-            // Relative time
-            onoff = (unsigned int)rint(1000.0 * dblValue);
-            onoff += lastEvent;
+
+         OnOffEvent event;
+         event.RandNumGen    = 0x00;
+         event.RelativeTime  = false;
+         for(size_t i = 0; i < sizeof(event.ValueArray) / sizeof(event.ValueArray[0]); i++) {
+            event.ValueArray[i] = 0.0;
          }
-         else if(sscanf((const char*)&parameters[m], "%lf%n", &dblValue, &n) == 1) {
-            // Absolute time
-            onoff = (unsigned int)rint(1000.0 * dblValue);
+         if(parameters[m] == '+') {
+            event.RelativeTime = true;
+            m++;
+         }
+         else if(strncmp((const char*)&parameters[m], "repeat", 6) == 0) {
+            m += 6;
+            trafficSpec.RepeatOnOff = true;
+            break;
+         }
+         const char* p = parseNextEntry((const char*)&parameters[m], (double*)&event.ValueArray, (uint8_t*)&event.RandNumGen);
+         if(p != NULL) {
+            m += (long)p - (long)&parameters[m + 1];
          }
          else {
-            cerr << "ERROR: Invalid on/off list at " << (const char*)&parameters[m] << "!" << std::endl;
+            m += strlen((const char*)&parameters[m]);
+         }
+
+         trafficSpec.OnOffEvents.push_back(event);
+      }
+      if(trafficSpec.RepeatOnOff == true) {
+         if((trafficSpec.OnOffEvents.size() % 2) != 0) {
+            cerr << "ERROR: Repeated on/off traffic needs even number of events!" << endl;
             exit(1);
          }
-         trafficSpec.OnOffEvents.insert(onoff);
-         lastEvent = onoff;
-         m += n;
+         for(std::vector<OnOffEvent>::const_iterator iterator = trafficSpec.OnOffEvents.begin();
+            iterator != trafficSpec.OnOffEvents.end();iterator++) {
+            if((*iterator).RelativeTime == false) {
+               cerr << "ERROR: Repeated on/off traffic needs relative times!" << endl;
+               exit(1);
+            }
+         }
       }
       n = m;
    }
@@ -414,6 +440,26 @@ static const char* parseTrafficSpecOption(const char*      parameters,
       trafficSpec.PathMgr = std::string((const char*)&pathMgr);
       n = 8 + strlen((const char*)&pathMgr);
    }
+   else if(strncmp(parameters, "cc=", 3) == 0) {
+      char   congestionControl[NETPERFMETER_CC_LENGTH + 1];
+      size_t i = 0;
+      while(i < NETPERFMETER_CC_LENGTH) {
+         if( (parameters[3 + i] == ':') ||
+             (parameters[3 + i] == 0x00) ) {
+            break;
+         }
+         congestionControl[i] = parameters[3 + i];
+         i++;
+      }
+      congestionControl[i] = 0x00;
+      if( (parameters[3 + i] != ':') && (parameters[3 + i] != 0x00) ) {
+         cerr << "ERROR: Invalid \"pathmgr\" setting: " << (const char*)&parameters[8]
+              << " - name too long!" << std::endl;
+          exit(1);
+      }
+      trafficSpec.CongestionControl = std::string((const char*)&congestionControl);
+      n = 3 + strlen((const char*)&congestionControl);
+   }
    else {
       cerr << "ERROR: Invalid option \"" << parameters << "\"!" << endl;
       exit(1);
@@ -806,31 +852,11 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
    gMPTCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_MPTCP, localPort - 1,
                                       gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
    if(gMPTCPSocket < 0) {
-      cerr << "ERROR: Failed to create and bind MPTCP socket - "
-           << strerror(errno) << "!" << endl;
-      exit(1);
-   }
-
-// FIXME! Add proper, platform-independent code here!
-#ifndef __linux__
-#warning MPTCP is currently only available on Linux!
-#else
-   int cmtOnOff = 1;
-   if(ext_setsockopt(gMPTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-      cerr << "NOTE: Compiled with MPTCP support, but unable to initialize it: "
-           << strerror(errno) << "!" << endl;
-      ext_close(gMPTCPSocket);
-      gMPTCPSocket = -1;
-   }
-   else {
-      cmtOnOff = 0;
-      if(ext_setsockopt(gTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-         cerr << "ERROR: Failed to disable MPTCP on TCP socket - "
-              << strerror(errno) << "!" << endl;
-         exit(1);
+      if(gOutputVerbosity >= NPFOV_STATUS) {
+         cerr << "NOTE: Unable to create and bind MPTCP socket - "
+            << strerror(errno) << "!" << endl;
       }
    }
-#endif
 #endif
 
    gUDPSocket = createAndBindSocket(AF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP, localPort,
@@ -884,10 +910,12 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
 
    // ====== Print status ===================================================
-   cout << "Passive Mode: Accepting TCP"
-        << ((gMPTCPSocket > 0) ? "+MPTCP" : "")
-        << "/UDP/SCTP" << ((gDCCPSocket > 0) ? "/DCCP" : "")
-        << " connections on port " << localPort << endl << endl;
+   if(gOutputVerbosity >= NPFOV_STATUS) {
+      cout << "Passive Mode: Accepting TCP"
+           << ((gMPTCPSocket > 0) ? "+MPTCP" : "")
+           << "/UDP/SCTP" << ((gDCCPSocket > 0) ? "/DCCP" : "")
+           << " connections on port " << localPort << endl << endl;
+   }
 
 
    // ====== Main loop ======================================================
