@@ -1,7 +1,7 @@
-/* $Id: netperfmeterpackets.h 1877 2014-07-09 13:44:24Z dreibh $
+/* $Id: netperfmeterpackets.h 1933 2014-11-17 07:41:37Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2014 by Thomas Dreibholz
+ * Copyright (C) 2009-2015 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,44 +70,57 @@ struct NetPerfMeterAcknowledgeMessage
 #define NETPERFMETER_RNG_INPUT_PARAMETERS  4
 
 #define NETPERFMETER_PATHMGR_LENGTH       16
+#define NETPERFMETER_CC_LENGTH            16
+
+struct NetPerfMeterOnOffEvent
+{
+   uint8_t          RandNumGen;
+   uint8_t          Flags;
+   uint16_t         pad;
+   network_double_t ValueArray[NETPERFMETER_RNG_INPUT_PARAMETERS];
+};
+
+#define NPOOEF_RELTIME (1 << 0)
 
 struct NetPerfMeterAddFlowMessage
 {
-   NetPerfMeterHeader   Header;
+   NetPerfMeterHeader     Header;
 
-   uint32_t             FlowID;
-   uint64_t             MeasurementID;
-   uint16_t             StreamID;
-   uint8_t              Protocol;
-   uint8_t              pad;
+   uint32_t               FlowID;
+   uint64_t               MeasurementID;
+   uint16_t               StreamID;
+   uint8_t                Protocol;
+   uint8_t                pad;
 
-   char                 Description[NETPERFMETER_DESCRIPTION_SIZE];
+   char                   Description[NETPERFMETER_DESCRIPTION_SIZE];
 
-   uint32_t             OrderedMode;
-   uint32_t             ReliableMode;
-   uint32_t             RetransmissionTrials;
+   uint32_t               OrderedMode;
+   uint32_t               ReliableMode;
+   uint32_t               RetransmissionTrials;
 
-   network_double_t     FrameRate[NETPERFMETER_RNG_INPUT_PARAMETERS];
-   network_double_t     FrameSize[NETPERFMETER_RNG_INPUT_PARAMETERS];
-   uint8_t              FrameRateRng;
-   uint8_t              FrameSizeRng;
+   network_double_t       FrameRate[NETPERFMETER_RNG_INPUT_PARAMETERS];
+   network_double_t       FrameSize[NETPERFMETER_RNG_INPUT_PARAMETERS];
+   uint8_t                FrameRateRng;
+   uint8_t                FrameSizeRng;
 
-   uint32_t             RcvBufferSize;
-   uint32_t             SndBufferSize;
+   uint32_t               RcvBufferSize;
+   uint32_t               SndBufferSize;
 
-   uint16_t             MaxMsgSize;
-   uint8_t              CMT;
-   uint8_t              CCID;
+   uint16_t               MaxMsgSize;
+   uint8_t                CMT;
+   uint8_t                CCID;
 
-   uint16_t             NDiffPorts;
-   char                 PathMgr[NETPERFMETER_PATHMGR_LENGTH];
+   uint16_t               NDiffPorts;
+   char                   PathMgr[NETPERFMETER_PATHMGR_LENGTH];
+   char                   CongestionControl[NETPERFMETER_CC_LENGTH];
 
-   uint16_t             OnOffEvents;
-   uint32_t             OnOffEvent[];
+   uint16_t               OnOffEvents;
+   NetPerfMeterOnOffEvent OnOffEvent[];
 } __attribute__((packed));
 
-#define NPMAFF_DEBUG                     (1 << 0)
-#define NPMAFF_NODELAY                   (1 << 1)
+#define NPMAFF_DEBUG         (1 << 0)
+#define NPMAFF_NODELAY       (1 << 1)
+#define NPMAFF_REPEATONOFF   (1 << 2)
 
 // RetransmissionTrials in milliseconds (highest bit of 32-bit value set)
 #define NPMAF_RTX_TRIALS_IN_MILLISECONDS (1 << 31)
